module OED

  EntryListDir = '/Tables'

  EntryListBaseName = 'entry'
  DateListBaseName = 'lbd'
  ListIndexExt = '.idx'
  ListDataExt = '.tbl'

  def quoteDate(date)
    year = date / 1000
    type = (date % 1000) / 10
    dots = date % 10

    return '' if year == 3000

    case type
    when  0..2  then t = 'a'
    when  3..5  then t = '?a'
    when  6..8  then t = ''
    when  9..11 then t = '?'
    when 12..14 then t = 'c'
    when 15..17 then t = '?c'
    else             t = ''
    end

    year = sprintf('%d', year)
    dots.times do |n|
      year[-(n + 1)] = '.'
    end

    return t + year
  end

  module_function :quoteDate

### List by entry
class EntryList
  include Enumerable

  def initialize(path, basename = EntryListBaseName)
    path = path.sub(/[\/\\]$/, '') + '/'
    idxfname = path + basename + ListIndexExt
    datafname = path + basename + ListDataExt
    @idxf = open(idxfname, 'rb')
    @dataf = open(datafname, 'rb')

    @size = File::size(idxfname) / 4 - 2
  end

  def size
    @size
  end

  ### n > 0
  def get(n)
    return nil if (n < 1) or (n > @size)

    @idxf.seek(4 * (n - 1))
#    @idxf.seek(4 * (@size - n))
    pos, nextpos = @idxf.read(8).unpack('VV')

    @dataf.seek(pos)
    rec = @dataf.read(nextpos - pos)
    date, docNum, bodyCount, dateNum, edition, title = rec.unpack('VVvVCa*')
    return title, date, docNum, bodyCount, dateNum, edition
  end

  def each
    1.upto(@size) do |n|
      yield get(n)
    end
  end

  def reverse_each
    @size.downto(1) do |n|
      yield get(n)
    end
  end

  def close
    @idxf.close
    @dataf.close
  end

end

### List by date
class DateList < EntryList

  def initialize(path, basename = DateListBaseName)
    super
  end

  ### n > 0
  def get(n)
    return nil if (n < 1) or (n > @size)

    @idxf.seek(4 * (n - 1))
#    @idxf.seek(4 * (@size - n))
    pos, nextpos = @idxf.read(8).unpack('VV')
    @dataf.seek(pos)
    rec = @dataf.read(nextpos - pos)
    date, entryNum, title = rec.unpack('VVa*')
    return title, date, entryNum
  end

end

end ##### module ################################################

if __FILE__ == $0

  include OED

def printListByEntry(path)

  elist = EntryList::new(path)

  puts 'Entry#----Date----Doc#----N--Date#-E-Headword---'

  count = ($DEBUG) ? 10 : elist.size
  n = 0
  count.times do |n|
    STDERR.print n, "\r" if n % 1000 == 0
    num = elist.size - n
#    num = n + 1
    title, date, docNum, docCount, dateNum, edition = elist.get(num)
    puts '%06d: %6s %7d %4d %6d %c %s' % [num, quoteDate(date), docNum, docCount, dateNum, edition, title]
  end

  STDERR.print n + 1, "\n"
  elist.close

end

def printListByDate(path)

  elist = DateList::new(path)

  puts 'Date#-----Date-Entry#-Headword---'

  count = ($DEBUG) ? 10 : elist.size
  n = 0
  1.upto(count) do |n|
    STDERR.print n, "\r" if n % 1000 == 0
    title, date, entryNum = elist.get(n)
    puts '%06d: %6s %6d %s' %  [n, quoteDate(date), entryNum, title]
  end

  STDERR.print n, "\n"
  elist.close

end

def printWordOfDay(path, wordfile)

  elist = EntryList::new(path)

  open(wordfile, 'r') do |f|
    puts 'Entry#----Date----Doc#----N--Date#-E-Headword---'
    while line = f.gets
      if line =~ /(\d+)/
        entnum = $1.to_i
        title, date, docNum, docCount, dateNum, edition = elist.get(entnum)
        puts '%06d: %6s %7d %4d %6d %c %s' % [entnum, quoteDate(date), docNum, docCount, dateNum, edition, title]
      end
      break if $DEBUG && f.lineno >= 10
    end
  end

  elist.close

end

  home = ARGV.shift
  home = home.gsub("\\", '/').sub(/\/$/, '') + '/'

  printListByEntry(home + EntryListDir)
#  printListByDate(home + EntryListDir)
#  printWordOfDay(home + EntryListDir, home + '/bin/word.day')

end

